import merlin.util.*;
import merlin.filter.*;
import java.awt.*;
import java.util.*;

/** 
 * The Code Generator suprisingly generates code. It reads all files given on
 * the command line and opens these pattern-files. It then writes the class
 * Samples in merlin/util/Samples.java so that there is no need to
 * load any samples at startup. They are all compiled into the
 * application. 
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 0.9
 */

public class CodeGenerator 
{
    private static final int SIZE = 100;
    private static final int POINT_DISTANCE = 49;
    public static void main(String[] argv)
	{
	    Vector numeral = new Vector();
	    Vector extended = new Vector();
	    Vector natural = new Vector();
	    for (int i = 0; i < argv.length; i++)
	    {
		Pattern p = new Pattern();
		p.loadStrokes(argv[i]);
		if (argv[i].indexOf("numeral") != -1)
		    numeral.addElement(p);
		else
		    if (argv[i].indexOf("extended") != -1)
			extended.addElement(p);
		    else
			if (argv[i].indexOf("natural") != -1)
			    natural.addElement(p);
	    }

	    System.out.println("package merlin.util;\n");
	    System.out.println("/** This code was generated by CodeGenerator" +
			       ". Do not edit.");
	    System.out.println(" * @author CodeGenerator");
	    System.out.println(" * @version 1.0");
	    System.out.println(" */");

	    System.out.println("public class Samples\n{");
	    generateCode(numeral, "numeral");
	    generateCode(extended, "extended");
	    generateCode(natural, "natural");
	    System.out.println("}");
	}
    
    private static void generateCode(Vector v, String name)
	{
	    NormFilter nf = new NormFilter(SIZE, SIZE);
	    SmoothFilter sf = new SmoothFilter();
	    PointDistanceNormFilter psnf = 
		new PointDistanceNormFilter(POINT_DISTANCE);
	    Vector prep = new Vector();
	    prep.addElement(nf);
	    prep.addElement(sf);
	    prep.addElement(psnf);
	    FilterCollection preprocessor = new FilterCollection(prep);

	    System.out.print("    public static final char " + name + 
			       "Ascii"+"[]= {");
	    for (int i = 0; i < v.size(); i++)
	    {
		char ascii = ((Pattern) v.elementAt(i)).ascii;
		System.out.print((int) ascii);
		if (i < v.size()-1)
		    System.out.print(", ");
	    }
	    System.out.println("};\n");

	    System.out.println("    public static final byte " + name + 
			       "[][][] = {");
	    for (int i = 0; i < v.size(); i++)
	    {
		Pattern p = (Pattern) v.elementAt(i);
		p = preprocessor.apply(p);
		// Start of new pattern
		System.out.print("        {");
		for (Enumeration en = p.strokes(); en.hasMoreElements(); )
		{
		    // Start of new stroke in pattern
		    Vector points = (Vector) en.nextElement();
		    for (int k = 0; k < points.size(); k++)
		    {
			Point pn = (Point) points.elementAt(k);
			System.out.print("{" + pn.x + "," + pn.y + "}");
			if (k < points.size()-1 || en.hasMoreElements())
			    System.out.print(",");
		    }
		}
		System.out.print("}");
		if (i < v.size()-1)
		    System.out.println(",");
	    }
	    System.out.println();
	    System.out.println("    };\n");
	}
}

    


