import merlin.util.*;
import merlin.filter.*;
import merlin.wizard.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * The recognition application. 
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 *
 * modified by Marcus Wolschon to be used in project jAugment.
 */
public class Merlin extends Component 
    implements TimeOutable, MouseListener, MouseMotionListener
{
 public static interface RecogListener
 {
  public void characterRecognized(char c);
 };

 private RecogListener rl = null;
 public void addRecogListener(RecogListener rl2)
 {this.rl = rl2;};

 private void fireCR(char c)
 {
  if(rl != null)
    rl.characterRecognized(c);

  System.out.print("recognized: "+c);  
 }

    private static final int TIMEOUT = 500;
    public static final int SINGLE_STROKE =   0;
    public static final int MULTIPLE_STROKE = 1;
    private static final byte DRAG =     0;
    private static final byte PEN_UP =   1;
    private static final byte PEN_DOWN = 2;
    private static final byte BUTTON1_PRESSED =   3;
    private static final byte BUTTON2_PRESSED =   4;
    private static final byte BUTTON3_PRESSED =   5;
    private static final byte BUTTON1_RELEASED =  6;
    private static final byte BUTTON2_RELEASED =  7;
    private static final byte BUTTON3_RELEASED =  8;
    private static final char SHIFT_SYMBOL = '.';

    /** 
     * The main fuction
     * @param argv The first string of argv should be the file to read 
     * mouse eventsfrom (normally /dev/stdin). The second string of argv is
     * the file to output recognized characters on (normally /dev/stdout)
     */
    public static void main(String[] argv)
	{
	    Merlin m = new Merlin();
	    Frame f = new Frame( "Merlin 1.0" );
	    f.add( m );
	    f.show();
	    Insets i = f.getInsets();
	    f.setSize( 320 + i.left + i.right, 200 + i.top + i.bottom );

	    try
	    {
		DataInputStream dis = 
		    new DataInputStream( System.in );

		Thread.currentThread().setPriority( Thread.MIN_PRIORITY );
		
		while (true)
		{
		    if (dis.available() >= 9)
			{
			    byte com = dis.readByte();
			    
			    int x = 160 + (int) (320.0 * 
						 ((float) dis.readInt() / 500));
			    int y = 100 + (int) (200.0 * 
						 ((float) dis.readInt() / 356));
			    
			    MouseEvent me = 
				new MouseEvent(m, 0, 0, 0, x, y, 0, false); 
			    switch (com)
				{
				case DRAG: m.mouseDragged(me); break;
				case PEN_UP: m.mouseReleased(me); break;
				case PEN_DOWN: m.mousePressed(me); break;
				case BUTTON1_PRESSED: 
				    // System.out.println("*click 1*");
				    m.clear(); 
				    m.setMode(SINGLE_STROKE);
				    m.setBank(Wizard.NATURAL);
				    break;
				case BUTTON2_PRESSED: 
				    // System.out.println("*click 2*");
				    m.clear(); 
				    m.setMode(SINGLE_STROKE);
				    m.setBank(Wizard.NUMERAL);
				    break;
				case BUTTON3_PRESSED: 
				    // System.out.println("*click 3*");
				    // m.clear(); 
				    // m.setMode(MULTIPLE_STROKE);
				    // m.setBank(Wizard.EXTENDED);
				    break;
				default:
				    System.out.println("* Unknown event *");
				}
			}
		}
	    }
	    catch (IOException e)
	    {
		System.out.println("I/O Error in Merlin: " + e.getMessage());
	    }
	}


    /**
     * Constructor
     */
    public Merlin()
	{
	    addMouseListener( this );
	    addMouseMotionListener( this );
	    lastWasShift = false;
	    mode = SINGLE_STROKE;
	    pat = new Pattern();
	    curStroke = new Vector();
	    timer = null;
	    wiz = new ElasticWizard();
	    isTimedout = true;
	}

    /**
     * From Timeoutable interface. This is called by a Timer
     * when the user has been inactive for some time
     */
    public void timedOut()
	{
	    char c = wiz.recognize(pat);
	    if (mode == MULTIPLE_STROKE)
		restoreBank();
	    isTimedout = true;

	    if (c == 500)
	    {
		storeBank();
		setMode(MULTIPLE_STROKE);
		setBank(Wizard.EXTENDED);
	    }
	    else   
		if (c == SHIFT_SYMBOL)
		{
		    if (!lastWasShift)
			lastWasShift = true;
		    else
		    {
			lastWasShift = false;
			//fireCR(SHIFT_SYMBOL); 
			//System.out.print(SHIFT_SYMBOL);
		    }
		}
		else
		{
		    if (lastWasShift)
		    {
			lastWasShift = false;
			c = Character.toUpperCase(c);
		    }
		     fireCR(c);
		    //System.out.print(c);
		}
	    last = pat;
	}
    
    private void restoreBank()
	{
	    setMode(SINGLE_STROKE);
	    setBank(oldBank);
	}

    private void storeBank()
	{
	    oldBank = getBank();
	}
    /**
     * paint function
     **/
    public void paint( Graphics g )
    {
	pat.paint( g );
	Vector v = (Vector) curStroke;
	if ( v.size() > 1 )
	    {
		Point curp = (Point) v.firstElement();
		for ( int j = 1; j < v.size(); j++ )
		    {
			Point newp = (Point) v.elementAt( j );
			g.drawLine( curp.x, curp.y, newp.x, newp.y );
			curp = newp;
		    }
	    }
    }

    public void mouseEntered( MouseEvent me ) {}
    public void mouseExited( MouseEvent me ) {}
    public void mouseClicked( MouseEvent me ) {}
    public void mouseMoved( MouseEvent me ) {}

    /**
     * Called when the mouse is pressed
     */
    public void mousePressed(MouseEvent e)
	{
	    if (timer != null)
	    {
		timer.kill();
		timer.stop();
		timer = null;
	    }
	    if ( isTimedout )
		{
		    pat = new Pattern();
		    isTimedout = false;
		}
	    Point p = e.getPoint();
	    curStroke.addElement(p);
	}
    
    /**
     * Called when mouse is released
     */
    public void mouseReleased(MouseEvent e)
	{
	    Point p = e.getPoint();
	    curStroke.addElement(p);
	    pat.addStroke(curStroke);
	    curStroke = new Vector();
	    if (mode == SINGLE_STROKE)
		timedOut();
	    else
	    {
		timer = new Timer(this, TIMEOUT);
		timer.start();
	    }
	}
    
    /**
     * Called when mouse is dragged
     */
    public void mouseDragged(MouseEvent e)
	{
	    Point p = e.getPoint();
	    curStroke.addElement(p);
	    repaint();
	}
    
    /**
     * To get the current pattern
     * @return the pattern currently on the Merlin
     */
    public Pattern getPattern()
	{
	    return pat;
	}
    
    /**
     * To clear current pattern
     */
    public void clear()
	{
	   if (timer != null)
	   {
	       timer.kill();
	       timer.stop();
	       timer = null;
	   }
	   pat = new Pattern();
	   curStroke = new Vector();
	}
    
    /**
     * @return The last pattern drawn
     */
    public Pattern getLast()
	{
	    return last;
	}

    /**
     * Sets the wizard
     * @param wiz The new wizard to use
     */
    public void setWizard(Wizard wiz)
	{
	    this.wiz = wiz;
	}

    /**
     * Sets the mode
     * @param mode The new mode
     */
    public void setMode(int mode)
	{
	    this.mode = mode;
	}

    /**
     * Gets the mode
     * @return The mode
     */
    public int getMode()
	{
	    return mode;
	}

    /**
     * Sets the bank
     * @param bank The new mode
     */
    public void setBank(byte bank)
	{
	    wiz.setBank(bank);
	}

    /**
     * Gets the bank
     * @return The bank
     */
    public byte getBank()
	{
	    return wiz.getBank();
	}


    private boolean lastWasShift;
    private Pattern last;
    private Timer timer;
    private Pattern pat;
    private Vector curStroke;
    private Wizard wiz;
    private int mode;
    private byte oldBank;
    private boolean isTimedout;
}






