package merlin.filter;

import merlin.util.*;
import java.util.*;

/**
 * The representation of a FilterCollection. A FilterCollection is a filter
 * that is a made of a collection of filters. Filters that will be applied 
 * in a certain order.
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 */

public class FilterCollection extends Filter
{
    /**
     * Class constructor.
     */
    public FilterCollection()
	{
	}

    /**
     * Class constructor specifying the filters to use and the order. 
     * The filters are placed in a vector with the filter to be applied first
     * at the first position in the vector.
     * @param fs  The vecor of filters to use
     */
    public FilterCollection(Vector fs)
	{
	    this();
	    filters = fs;
	}

    /**
     * Sets filters that the FilterCollection will use
     * @param fs A vector of filters. 
     * element in the vector.
     */
    public void setFilters(Vector fs)
	{
	    filters = fs;
	}

    /**
     * Applies the filters on a Pattern
     * @param p The pattern to preprocess. The filters will be applied in 
     * the order that they are in the vector starting with the first element
     * @return The new preprocessed pattern
     */
    public Pattern apply(Pattern p)
	{
	    for (Enumeration en = filters.elements(); en.hasMoreElements();)
	    {
		Filter f = (Filter) en.nextElement();
		p = f.apply(p);
	    }
	    return p;
	}

    private Vector filters;
}

	    
	    
