package merlin.filter;

import merlin.util.*;
import java.awt.*;
import java.math.*;
import java.util.*;

/**
 * A normalizing filter. The character is normalized to fit into a fixed 
 * square
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 */

public class NormFilter extends Filter
{
    /** 
     * Constructor
     * @param width The width of the new area the char will be squezed into
     * @param height The height of the new area the char will be squezed into
     */
    public NormFilter(int width, int height)
	{
	    this.width = width-1;
	    this.height = height-1;
	}
    /**
     * You apply a filter to a pattern and get a new pattern
     * @param p  The pattern to apply the filter on
     * @return A new Pattern
     */
    public Pattern apply(Pattern pat)
	{
	    int xmin, ymin, xmax, ymax;
	    double xm, ym, dx, dy, delta, centerx, centery;

	    /* find the bounding box */
	    
	    if (!pat.strokes().hasMoreElements())
		return pat;
	    Point tmp = (Point) ((Vector) pat.strokes().nextElement()).firstElement();
	    xmin = xmax = tmp.x;
	    ymin = ymax = tmp.y;
	    
	    for (Enumeration str = pat.strokes(); str.hasMoreElements();)
		for (Enumeration pts = ((Vector) str.nextElement()).elements(); 
		     pts.hasMoreElements();)
		{
		    Point p = (Point) pts.nextElement();
		    if (p.x < xmin)
			xmin = p.x;
		    if (p.y < ymin)
			ymin = p.y;
		    if (p.x > xmax)
			xmax = p.x;
		    if (p.y > ymax)
			ymax = p.y;
		}
		
	    /* Center of gravity */
	    xm = (xmax + xmin)/2.0;
	    ym = (ymax + ymin)/2.0;

	    /* the spreads */
	    dx = (xmax - xmin)/2.0;
	    dy = (ymax - ymin)/2.0;
	    
	    delta = Math.max(dx, dy);

	    /* Start to transform coordinates */
	    centerx = width/2.0;
	    centery = height/2.0;
	    
	    Pattern pat2 = new Pattern();
	    for (Enumeration str = pat.strokes(); str.hasMoreElements();)
	    {
		Vector stroke = new Vector();
		Vector pts = (Vector) str.nextElement();
		for (int i = 0; i < pts.size(); i++) 
		{
		    Point p = (Point) pts.elementAt(i);
		    Point newp = new Point();
		    newp.x = (int )(centerx + centerx*((p.x-xm)/delta));
		    newp.y = (int)(centery + centery*((p.y-ym)/delta));
		    stroke.addElement(newp);
		}
		pat2.addStroke(stroke);
	    }
	    pat2.ascii = pat.ascii;
	    return pat2;	    
	}
    private int width, height;
}


