package merlin.filter;

import merlin.util.*;
import java.awt.*;
import java.lang.*;
import java.util.*;

/**
 * A point distance normalisation filter. 
 * Points lying closer than the specified threshold are removed
 * to reduce redundancy and increase computational speed 
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 */

public class PointDistanceNormFilter extends Filter
{
    /** 
     * Default constructor
     * @param threshold The smallest distance (in pixel units) between points
     */
    public PointDistanceNormFilter(int threshold)
	{
	    this.threshold = threshold;
	}

    /**
     * You apply a filter to a pattern and get a new pattern
     * @param p  The pattern to apply the filter on
     * @return A new Pattern
     */
    public Pattern apply(Pattern pat)
	{
	    Pattern pat2 = new Pattern();
	    for (Enumeration strokes = pat.strokes();
		 strokes.hasMoreElements();)
	    {
		Vector points = (Vector) strokes.nextElement();
		Vector newStroke = new Vector();
		for (int index = 0; index < points.size(); index++)
		{
		    if (index == 0)
			newStroke.addElement(points.elementAt(index));
		    else
			if (distance((Point) newStroke.lastElement(),
				     (Point) points.elementAt(index)) 
			    >= threshold)
			    newStroke.addElement(points.elementAt(index));
		}
		pat2.addStroke(newStroke);
	    }
	    pat2.ascii = pat.ascii;
	    return pat2;	    
	}
    
    private int distance(Point p1, Point p2)
	{
	    return (p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y);
	}
	   
    private int threshold;
}


