package merlin.util;

import merlin.filter.*;
import java.util.*;
import java.awt.*;
import java.io.*;

/**
 * The representation of a pattern.
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 */

public class Pattern
{
    static private final boolean DEBUG = false;
    /**
     * Class constructor.
     */
    public Pattern()
	{
	    strokes_ = new Vector();
	}

    /**
     * paint a pattern
     */
    public void paint( Graphics g )
    {
	g.setColor( Color.black );
	for ( int i = 0; i < strokes_.size(); i++ )
	    {
		Vector v = (Vector) strokes_.elementAt( i );
		if ( v.size() > 1 )
		    {
			Point curp = (Point) v.firstElement();
			for ( int j = 1; j < v.size(); j++ )
			    {
				Point newp = (Point) v.elementAt( j );
				g.drawLine( curp.x, curp.y, newp.x, newp.y );
				curp = newp;
			    }
		    }
	    }	    	
    }

    /**
     * Adds a stroke. A stroke is represented as a Vector of Points, 
     * starting with the Point first drawn.
     * @param s The stroke
     */
    public void addStroke(Vector s)
	{
	    strokes_.addElement(s);
	}
    
    /**
     * Adds strokes from a file. 
     * The format of the file is the format used by Rob Kassel in his
     * PhD-thesis:<BR> <I>
     * <A HREF="http://www.testbed.era.ericsson.se/~erasshe/reports.html">
     * A comparison of Approaches to On-line Handwritten Character Recognition
     * </A></I> 
     * @param s The filename
     */
    public void loadStrokes(String filename)
	{
	    try
	    {
		FileInputStream fr = new FileInputStream(filename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fr));
		String line;
		boolean readingStroke = false;
		Vector cur = new Vector();
		int x, y;

		while ((line = in.readLine()) != null)
		{
		    if (line.startsWith(".COMMENT"))
		    {
			if (line.indexOf("Prompt") != -1)
			{
			    int index = line.indexOf("\"");
			    if (index == -1)
				System.out.println("Error: Pattern");
			    ascii = line.charAt(index + 1);
			    if (ascii == '#')
			    {
				char[] tmp = new char[5];
				int i = index + 2;
				char c;
				while (Character.isDigit(c = line.charAt(i)))
				{
				    tmp[i - index - 2] = c;
				    i++;
				}
				String stmp = new String(tmp, 0, i - 2 - index);
				try
				{
				    ascii = (char) Integer.parseInt(stmp);
				}
				catch (NumberFormatException nfe)
				{
				    System.out.println(nfe.getMessage());
				    System.out.println("No number found");
				}
			    }   
			}
		    }
		    if (!readingStroke)
		    {
			if (line.startsWith(".PEN_DOWN"))
			{
			    readingStroke = true;
			}
		    }
		    else
		    {
			if (line.length() > 0 && !Character.isDigit(line.charAt(0)))
			{
			    readingStroke = false;
			    addStroke(cur);
			    cur = new Vector();
			}
			
			if (line.length() > 0 && Character.isDigit(line.charAt(0)))
			{
			    if (readingStroke)
			    {

				int index = 0;
				while (!Character.isSpaceChar(line.charAt(index)))
				    index++;
				String rest = line.substring(0, index).trim();
				x = Integer.parseInt(rest);
				while (Character.isSpaceChar(line.charAt(index)))
				    index++;
				int tmp = index;
				while (!Character.isSpaceChar(line.charAt(index)))
				    index++;
				rest = line.substring(tmp, index).trim();
				y = 3058 - Integer.parseInt(rest);
				cur.addElement(new Point(x, y));
			    }
			}
		    }
		}
		if (DEBUG)
		    System.out.println("Added " + strokes_.size() + 
				       " strokes from file " + filename);
		in.close();
		fr.close();
	    }
	    catch(FileNotFoundException fnfe)
	    {
		System.out.println(fnfe.getMessage());
	    }
	    catch(IOException ioe)
	    {
		System.out.println(ioe.getMessage());
	    }
	    catch(NumberFormatException nfe)
	    {
		System.out.println("Ouch");
		System.out.println(nfe.getMessage());
	    }
	}

    /**
     * Store pattern on a file. 
     * The format of the file is the format used by Rob Kassel in his
     * PhD-thesis:<BR> <I>
     * <A HREF="http://www.testbed.era.ericsson.se/~erasshe/reports.html">
     * A comparison of Approaches to On-line Handwritten Character Recognition
     * </A></I> 
     * @param s The filename
     */
    public void storeOn(String filename)
	{
	    try
	    {
		int count = 0;
		FileWriter fw = new FileWriter(filename);
		String asciiString = Character.isLetterOrDigit(ascii) ? 
		    "" + ascii : 
		    "#" + (new Integer((int) ascii)).toString();
		
		String s = ".COMMENT Prompt string: \"" + asciiString + "\"\n\n"; 
		fw.write(s, 0, s.length());
		for (Enumeration en = strokes_.elements(); 
		     en.hasMoreElements();)
		{
		    s = ".PEN_DOWN\n";
		    fw.write(s, 0, s.length());
		    for (Enumeration en2 = 
			     ((Vector) en.nextElement()).elements(); 
			 en2.hasMoreElements();)
		    {
			Point p1 = (Point) en2.nextElement();
			s = "" + p1.x + " " + (3058-p1.y) + " " + count + "\n";
			fw.write(s, 0, s.length());
			count++;
		    }
		    s = ".PEN_UP\n\n";
		    fw.write(s, 0, s.length());
		}
		fw.close();
	    }
	    catch(IOException ioe)
	    {
		System.out.println(ioe.getMessage());
	    }
	}

    /**
     * Returns the strokes as an Enumeration.
     * @return The enumeration
     */
    public Enumeration strokes()
	{
	    return strokes_.elements();
	}
    
    /** 
     * The meaning of this pattern. What ascii character it represents
     */
    public char ascii;
    private Vector strokes_;
}

	    
	    
