package merlin.util;

import java.lang.*;
import java.util.*;

/**
 * The representation of a Timer. A timer runs in the background. 
 * The object given at creation is timed out after a certain time.
 * This is done by calling its timedOut-method
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 1.0
 */

public class Timer extends Thread
{
    /**
     * Class constructor.
     */
    public Timer(TimeOutable to, long time)
	{
	    Date now = new Date();
	    this.when = new Date(now.getTime() + time);
	    killed = false;
	    this.to = to;
	}

    /**
     * The timer waits the time and then calls the objects timedOut-function
     */
    public void run()
	{
	    while (when.after(new Date()))
		Thread.yield(); /* Wasting time */
	    if (!killed)
		to.timedOut();
	}
    
    /**
     * The timer is no longer wanted and is killed.
     */
    public void kill()
	{
	    killed = true;
	}

    /**
     * The timer is restarted
     * @param time The new time to wait
     */
    public void restart(long time)
	{
	    when = new Date((new Date()).getTime() + time);
	}

    TimeOutable to;
    private Date when;
    private boolean killed;
}

	    
	    
