package merlin.wizard;

import merlin.util.*;

/**
 * The interface of a Wizard. A Wizard takes a Pattern and 
 * produces a guess of what it looks like
 * Copyright (c) Stefan Hellkvist (see COPYRIGHT)
 * @author Stefan Hellkvist
 * @version 0.9
 */

public abstract class Wizard
{
    public static final byte NATURAL = 0;
    public static final byte NUMERAL = 1;
    public static final byte EXTENDED = 2;
    
    /**
     * You give the Wizard a pattern and lets him recognize it.
     * You don't need to preprocess the pattern with any filter.
     * The Wizard will do all the necessary filtering.
     * @param p  The pattern to recognize
     * @return The ascii value of the pattern
     */
    public abstract char recognize(Pattern p);
    
    /**
     * Tell the wizard in what bank to look in
     * @param bank One of NATURAL, NUMERAL or EXTENDED
     */
    public void setBank(byte bank)
	{
	    this.bank = bank;
	}
    
    /**
     * Gets the current bank 
     * @return The current bank
     */
    public byte getBank()
	{
	    return bank;
	}

    private byte bank;
}
