#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "com.h"

#define DEBUG 0

static int aDev;
char version[15];
char version2[15];

void sendBreakToPad();

void 
resetPad()
{
    /* 
     * A reset is performed through toggling of RTS, and nothing else
     * Some magic is also performed here, though. I don't know why but
     * it started to work when I did it so...
     */
    int flags;
    ioctl(aDev, TIOCMGET, &flags);
    flags &= ~(TIOCM_RTS | TIOCM_DTR); /* Pull DTR low also. Why? I donno. */
    ioctl(aDev, TIOCMSET, &flags);
    usleep(1000); /* A magically determined sleep */
    flags |= TIOCM_RTS; /* I forget (on purpose) to pull up DTR again */
    ioctl(aDev, TIOCMSET, &flags);
    usleep(1000000); /* A magically determined sleep */
    tcflush(aDev, TCIOFLUSH); /* A flush is also performed for magic reasons */
}

/* 
 * A very bad way of converting two bytes to an int.
 * The reason why it's done this way is that it was done in this very bad
 * way in the example from the manufacturer written in the very bad language 
 * Visual Basic.
 * I haven't had the time to alter it though.
 */
int 
convert_to_binary(unsigned char num1, unsigned char num2)
{
    int Value = 0;

    /* if sign is positive convert bits to integer */
    if (!(num2 & 16))
    { 
        if (num1 & 1) Value = Value + 1;
        if (num1 & 2) Value = Value + 2;
        if (num1 & 4) Value = Value + 4;
        if (num1 & 8) Value = Value + 8;
        if (num1 & 16) Value = Value + 16;
        if (num1 & 32) Value = Value + 32;
        if (num1 & 64) Value = Value + 64;
        if (num2 & 1) Value = Value + 128;
        if (num2 & 2) Value = Value + 256;
        if (num2 & 4) Value = Value + 512;
        if (num2 & 8) Value = Value + 1024;
    }
    else
    {
        if (!(num1 & 1)) Value = Value + 1;
        if (!(num1 & 2)) Value = Value + 2;
        if (!(num1 & 4)) Value = Value + 4;
        if (!(num1 & 8)) Value = Value + 8;
        if (!(num1 & 16)) Value = Value + 16;
        if (!(num1 & 32)) Value = Value + 32;
        if (!(num1 & 64)) Value = Value + 64;
        if (!(num2 & 1)) Value = Value + 128;
        if (!(num2 & 2)) Value = Value + 256;
        if (!(num2 & 4)) Value = Value + 512;
        if (!(num2 & 8)) Value = Value + 1024;
        Value = -(Value + 1);
    }
    return Value;
}

void
readPadEvent(struct PadEvent *pe)
{
    unsigned int tmp;
    unsigned char buffer[6];
    int done = 0;
    
    while (!done)
    {
    	readCharsFromPad(1, buffer);
    	if (buffer[0] & 0x80)
    		done = 1;
    }
    
    readCharsFromPad(5, buffer+1);
    pe->rightButton = buffer[0] & 0x10;
    pe->leftButton = buffer[0] & 0x20;
    pe->virtualButton = buffer[0] & 0x08;
    pe->padInUse = buffer[0] & 0x04;
    pe->x = convert_to_binary(buffer[1], buffer[2]);
    pe->y = convert_to_binary(buffer[3], buffer[4]);
    pe->z = buffer[5]; 
}

void 
writeCharToPad(unsigned char one_ch)
{
    write(aDev, &one_ch, 1);
}

unsigned char
readCharFromPad()
{
    unsigned char c;
    read(aDev, &c, 1);
    return c;
}

void 
readCharsFromPad(int n, unsigned char *buffer)
{
    int i;
    for (i = 0; i < n; i++)
    {
	buffer[i] = readCharFromPad();
    }
}

void 
writeCharsToPad(int n, unsigned char *buffer)
{
    write(aDev, buffer, n);
}

void
initCom(speed_t speed, unsigned int nbits)
{
    struct termios tty;
    bzero(&tty, sizeof(struct termios));
    tty.c_cflag = nbits | CLOCAL | CREAD | CRTSCTS;
    tty.c_cc[VTIME] = 0;
    tty.c_cc[VMIN] = 1;
    cfsetispeed(&tty, speed);
    cfsetospeed(&tty, speed);
    tcsetattr(aDev, TCSANOW, &tty);
} 

int 
initPad(char *s)
{
   struct termios term;
   speed_t baud;
   char buffer[15];
   int i;

   /* Opens the device */
   if (DEBUG) { fprintf(stderr, "Opens device %s...", s); fflush(stdout); }
   if ((aDev = open(s, O_RDWR | O_NOCTTY)) < 0)
   {
      fprintf(stderr, "ERROR: Error opening device\n");
      exit(1);
   }
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }

   /* 
    * Resetting pad 
    */
   if (DEBUG) { fprintf(stderr, "Resetting pad..."); fflush(stdout); }
   resetPad();
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }
   
   /* 
    * Sets attributes and speed 
    * "1200,N,7,1" (1200 baud, no parity, 7 data bits, 1 stop bit
    */
   initCom(B1200, CS7);
   
   /* Send break to the pad */
   if (DEBUG) { fprintf(stderr, "Sends break to pad..."); fflush(stdout); }
   sendBreakToPad();
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }

   /* Read version string from pad */
   if (DEBUG) { fprintf(stderr, "Reads 7 chars..."); fflush(stdout); }
   readCharsFromPad(7, buffer);
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }
   buffer[7] = '\0';
   strcpy(version, buffer); 
   printf( "Version string is: \"%s\"\n", version );
   
   /* 
    * Sets attributes and speed 
    * "9600,N,8,1" (9600 baud, no parity, 8 data bits, 1 stop bit
    */
   initCom(B9600, CS8);
   if (DEBUG) { fprintf(stderr, "Sends break to pad..."); fflush(stdout); }
   sendBreakToPad();
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }

   /* Read version string from pad */
   if (DEBUG) { fprintf(stderr, "Reads 7 chars..."); fflush(stdout); }
   readCharsFromPad(7, buffer);
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }
   buffer[7] = '\0';
   strcpy(version2, buffer); 

   /* Sends break to the pad */
   if (DEBUG) { fprintf(stderr, "Sends break to pad..."); fflush(stdout); }
   sendBreakToPad();
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }
   
   /* Read prompt string from pad */
   if (DEBUG) { fprintf(stderr, "Reads 2 chars..."); fflush(stdout); }
   readCharsFromPad(2, buffer);
   if (DEBUG) {   fprintf(stderr, "done.\n"); fflush(stdout); }
   buffer[4] = '\0';

   /* Check if valid response */

/*   
   if (buffer[0] != '\0' || buffer[1] != '?')
   {
       fprintf(stderr, "Not a valid response (%c%c[%d %d])\n", 
	       buffer[0], buffer[1], buffer[0], buffer[1]);
       close(aDev);
       return -1;
   }
*/ 
   /* Send command "E" <CR> to enable pad */ 
   
   buffer[0] = 'E';
   buffer[1] = 13;
   if (DEBUG) { fprintf(stderr, "Writes 2 chars..."); fflush(stdout);}
   writeCharsToPad(2, buffer);
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }

   /* Read string from buffer to clear buffer */
   if (DEBUG) { fprintf(stderr, "Reads 2 chars..."); fflush(stdout); }
   readCharsFromPad(2, buffer); 
   if (DEBUG) { fprintf(stderr, "done.\n"); fflush(stdout); }

   /* Everything ok so return 0 */
   return 0;
}

void
sendBreakToPad()
{
    tcsendbreak(aDev, 2);
}
    
    






