#include <stdio.h>
#include "com.h"

#define TTY      "/dev/ttyS0"
#define DRAG     0
#define PEN_UP   1
#define PEN_DOWN 2
#define BUTTON1_PRESSED 3
#define BUTTON2_PRESSED 4
#define BUTTON3_PRESSED 5
#define BUTTON1_RELEASED 6
#define BUTTON2_RELEASED 7
#define BUTTON3_RELEASED 8


void
writeInt(FILE *stream, int k)
{
    char c;
    c = (char) (0xff & (k >> 24));
    fwrite(&c, 1, 1, stream);
    c = (char) (0xff & (k >> 16));
    fwrite(&c, 1, 1, stream);
    c = (char) (0xff & (k >> 8));
    fwrite(&c, 1, 1, stream);
    c = (char) (0xff & k);
    fwrite(&c, 1, 1, stream);
}

void
writeEvent(char type, int x, int y)
{
    fwrite(&type, 1, 1, stdout);
    writeInt(stdout, x);
    writeInt(stdout, y);
    fflush(stdout);
}

void 
printEvent(struct PadEvent *pe)
{
    fprintf(stderr, "RB: %d\n", pe->rightButton);
    fprintf(stderr, "LB: %d\n", pe->leftButton);
    fprintf(stderr, "VB: %d\n", pe->virtualButton);
    fprintf(stderr, "InUse: %d\n", pe->padInUse);
    fprintf(stderr, "(x, y, z) = (%d, %d, %d)\n", 
	    pe->x, pe->y, pe->z);
}

int
main()
{
    struct PadEvent event;
    int penUp = 1;
    //fprintf(stderr, "Merlin 0.9\n");
    //fprintf(stderr, "Initializing pad\n");
    if (initPad(TTY) == -1)
    {
	fprintf(stderr, "Pad not correctly initialized\n");
	exit(1);
    }
    //fprintf(stderr, "Pad initialized ok\n\n");
    
    while (1)
    {
	readPadEvent(&event);
	printEvent(&event);

	if (event.rightButton)
	{
	    writeEvent(BUTTON1_PRESSED, event.x, event.y);
	}

	if (event.leftButton)
	{
	    writeEvent(BUTTON2_PRESSED, event.x, event.y);
	}

	if (event.virtualButton)
	{
	    writeEvent(BUTTON3_PRESSED, event.x, event.y);
	}

	if (event.padInUse && penUp)
	{
	    penUp = 0;
	    writeEvent(PEN_DOWN, event.x, event.y);
	}
	else
	    if (!event.padInUse && !penUp)
	    {
		penUp = 1;
		writeEvent(PEN_UP, event.x, event.y);
	    }
	    else
		if (event.padInUse)
		{
		    writeEvent(DRAG, event.x, event.y);
		}
    }
    return 0;
}
