/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: char-rec
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-02-16, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *      - stop, restart,... super.* to properly fire events
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.charrec;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;
import java.net.URL;
import java.net.InetAddress;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;

import net.jini.core.entry.Entry;
import net.jini.core.discovery.LookupLocator;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
  //project
import jaugment2.jini.simpleService.simpleService;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;
import jaugment2.jini.LookupEventListener;
import jaugment2.jini.LookupEventEvent;


//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


public class CharrecService extends simpleService implements Remote
{

 private static final boolean debugMessages = false;
  //--------------------- initiaslisation&administration ------------------------------------------------------//

  private CharrecServiceProxy myproxy;


  /**
   * Load our state (e.g. attrubutes, service-id) from persistent storage
   */
  private void loadMe()
  {
      String filename = System.getProperty("jaugment2.charrec.CharrecService.persistentStateFilename");
      if(filename==null)
        {
         filename = "CharrecServiceState.dat";
         System.setProperty("jaugment2.charrec.CharrecService.persistentStateFilename", filename);
        };

      persistentState.setProperty("persistentStateFilename", filename);

   load();


   if(persistentState.get("attr")==null)
     {
      Entry[] attrSets = new Entry[]
                   {
                     new net.jini.lookup.entry.Name("CharrecService"),
                     new net.jini.lookup.entry.Comment("a service providing single-charrecter -recognition as an input-device"),
                     new net.jini.lookup.entry.ServiceInfo("CharrecService", "Marcus Wolschon <Suran@gmx.net>", "Marcus Wolschon <Suran@gmx.net>", "0.0.1", "standart", "00000"),

                     CharrecServiceRepresentation.getUIDescriptor()
                    };
      persistentState.put("attr", attrSets);
     };

   save();
  };


  public CharrecService() throws RemoteException
  {
   this(null);
  };

  public CharrecService(LookupLocator locallocator) throws RemoteException
  {
   if(debugMessages)Logger.debug.println("DEBUG: [CharrecService] creating myself...");
   try
     {

      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");

      myproxy = new CharrecServiceProxy(this);

      loadMe();

      Entry[] attrSets = (Entry[])persistentState.get("attr");

      this.init(myproxy, attrSets, this, locallocator);
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     };
  };



 /** restart this service once it has been stopped
   * @see stop
   */
 public void restart() throws Exception
 {
  super.restart();
 }

 /** stop this service. If exit is false all references to this
   * service are still valid but it has deregistered everywhere.
   * (this includes the RMI-activation-daemon)
   * @see restart
   * @see destroy
   * @param exit stop running
   */
 public void stop(boolean exit) throws Exception
 {
  super.stop(exit);
 }

 /** stop this service, destroy any persistent storage and exit
   */
 public void destroy() throws RemoteException
 {
  super.destroy();
 }





 //-------------------------- functionality -----------------------------------

}
